package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.content.Context;
import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Map;
import java.util.Objects;

import co.hyperverge.hypersnapsdk.activities.HVFaceActivity;
import co.hyperverge.hypersnapsdk.listeners.FaceCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVFaceConfig;
import co.hyperverge.hypersnapsdk.objects.HVResponse;


public class FLHVFaceCapture {

    static final String FACE_CAPTURE_START = "faceCaptureStart";
    static final String FACE_CAPTURE_SET_LIVENESS_ENDPOINT =
            "faceCaptureSetLivenessEndpoint";
    static final String FACE_CAPTURE_SET_SHOULD_RETURN_FULL_IMAGE_URL =
            "faceCaptureSetShouldReturnFullImageUrl";
    static final String FACE_CAPTURE_SET_SHOULD_SHOW_CAMERA_SWITCH_BUTTON =
            "faceCaptureSetShouldShowCameraSwitchButton";
    static final String FACE_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE =
            "faceCaptureSetShouldShowInstructionPage";
    static final String FACE_CAPTURE_SET_VIDEO_RECORDING =
            "faceCaptureSetVideoRecording";
    static final String FACE_CAPTURE_SET_LIVENESS_MODE =
            "faceCaptureSetLivenessMode";
    static final String FACE_CAPTURE_SET_FACE_CAPTURE_TITLE =
            "faceCaptureSetFaceCaptureTitle";
    static final String FACE_CAPTURE_SET_FACE_CAPTURE_SUBTITLE =
            "faceCaptureSetFaceCaptureSubtitle";
    static final String FACE_CAPTURE_SET_SHOULD_USE_BACK_CAMERA =
            "faceCaptureSetShouldUseBackCamera";
    static final String FACE_CAPTURE_SET_SHOULD_ADD_PADDING =
            "faceCaptureSetShouldAddPadding";
    static final String FACE_CAPTURE_SET_PADDING = "faceCaptureSetPadding";
    static final String FACE_CAPTURE_SET_LIVENESS_API_PARAMETERS =
            "faceCaptureSetLivenessAPIParameters";
    static final String FACE_CAPTURE_SET_LIVENESS_API_HEADERS =
            "faceCaptureSetLivenessAPIHeaders";
    static final String FACE_CAPTURE_SET_SHOULD_HANDLE_RETRIES =
            "faceCaptureSetShouldHandleRetries";
    static final String FACE_CAPTURE_SET_SHOULD_SHOW_CLOSE_ALERT =
            "faceCaptureSetShouldShowCloseAlert";
    static final String FACE_CAPTURE_SET_CLOSE_ALERT_DIALOG_TITLE =
            "faceCaptureSetCloseAlertDialogTitle";
    static final String FACE_CAPTURE_SET_CLOSE_ALERT_DIALOG_DESCRIPTION =
            "faceCaptureSetCloseAlertDialogDesc";
    static final String FACE_CAPTURE_SET_SHOW_TRUST_LOGOS =
            "faceCaptureSetShowTrustLogos";
    static final String FACE_CAPTURE_SET_UI_STRINGS = "faceCaptureSetUIStrings";
    static final String FACE_CAPTURE_SET_AUTO_CAPTURE = "faceCaptureSetAutoCapture";

    private static final String TAG = FLHVFaceCapture.class.getSimpleName();
    private HVFaceConfig faceConfig;

    public void setLivenessEndpoint(String livenessEndpoint) {
        getFaceConfig().setLivenessEndpoint(livenessEndpoint);
    }

    public HVFaceConfig getFaceConfig() {
        if (this.faceConfig == null) {
            this.faceConfig = new HVFaceConfig();
        }
        return this.faceConfig;

    }

    public void setShouldReturnFullImageUrl(Boolean shouldReturnFullImageUrl) {
        getFaceConfig().setShouldReturnFullImageUrl(shouldReturnFullImageUrl);
    }

    public void setClientID(String clientID) {
        getFaceConfig().setClientID(clientID);

    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void setCustomUIStrings(String customStrings) {
        try {
            if (customStrings != null && !customStrings.trim().isEmpty()) {
                getFaceConfig().setCustomUIStrings(new JSONObject(customStrings));
            }
        } catch (Exception e) {
            Log.e(TAG, Objects.requireNonNull(e.getMessage()));
        }
    }

    public void setShouldShowCameraSwitchButton(Boolean shouldShow) {
        getFaceConfig().setShouldShowCameraSwitchButton(shouldShow);
    }

    public void setShouldShowInstructionPage(Boolean shouldShowInstructionPage) {
        getFaceConfig().setShouldShowInstructionPage(shouldShowInstructionPage);
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void setLivenessMode(String livenessValue) {
        try {
            String liveness = livenessValue.split("\\.")[1];
            HVFaceConfig.LivenessMode livenessMode = HVFaceConfig.LivenessMode.valueOf(liveness);
            getFaceConfig().setLivenessMode(livenessMode);
        } catch (Exception exp) {
            Log.e(TAG, Objects.requireNonNull(exp.getMessage()));
        }
    }

    public void setVideoRecording(boolean shouldRecordVideo, int numberOfFrames, double bitrateM, int fps) {
        if (shouldRecordVideo) {
            getFaceConfig().setBitrateM(bitrateM);
            getFaceConfig().setNumberOfFrames(numberOfFrames);
            getFaceConfig().setFps(fps);
        }
        getFaceConfig().setShouldRecordVideo(shouldRecordVideo);
    }

    public void setAutoCapture(boolean shouldAutoCapture, int duration) {
        if (shouldAutoCapture) {
            getFaceConfig().setAutoCaptureDuration(duration);
        }
        getFaceConfig().setShouldAutoCapture(shouldAutoCapture);
    }

    public void setFaceCaptureTitle(String faceCaptureTitle) {
        getFaceConfig().setFaceCaptureTitle(faceCaptureTitle);
    }

    public void setFaceCaptureSubTitle(String faceCaptureSubTitle) {
        getFaceConfig().setFaceCaptureSubtitle(faceCaptureSubTitle);
    }

    public void setShouldUseBackCamera(Boolean shouldUseBackCamera) {
        getFaceConfig().setShouldUseBackCamera(shouldUseBackCamera);
    }


    public void setShouldEnableDataLogging(Boolean dataLogging) {
        getFaceConfig().setShouldEnableDataLogging(dataLogging);
    }

    public void setShouldAddPadding(Boolean shouldSetPadding) {
        getFaceConfig().setShouldEnablePadding(shouldSetPadding);

    }


    public void setPadding(double leftPadding, double rightPadding, double topPadding, double bottomPadding) {
        getFaceConfig().setPadding((float) (leftPadding), (float) rightPadding, (float) topPadding, (float) bottomPadding);
    }

    public void setLivenessAPIParameters(Map<String, Object> params) {
        JSONObject paramsObj = new JSONObject();
        try {
            if (params != null) {
                paramsObj = FLHVNetworkHelper.convertMapToJson(params);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        getFaceConfig().setLivenessAPIParameters(paramsObj);
    }

    public void setLivenessAPIHeaders(Map<String, Object> headers) {
        JSONObject headersObj = new JSONObject();
        try {
            if (headers != null) {
                headersObj = FLHVNetworkHelper.convertMapToJson(headers);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        getFaceConfig().setLivenessAPIHeaders(headersObj);
    }

    public void setShouldHandleRetries(boolean shouldHandleRetries) {
        getFaceConfig().setShouldHandleRetries(shouldHandleRetries);
    }

    public void setShouldShowCloseAlert(boolean shouldShowCloseAlert) {
        getFaceConfig().setShouldShowCloseAlert(shouldShowCloseAlert);
    }

    public void setCloseAlertDialogTitle(String closeAlertDialogTitle) {
        getFaceConfig().setCloseAlertDialogTitle(closeAlertDialogTitle);
    }

    public void setCloseAlertDialogDescription(String closeAlertDialogDescription) {
        getFaceConfig().setCloseAlertDialogDesc(closeAlertDialogDescription);
    }

    public void setShouldShowTrustLogos(boolean shouldShowTrustLogos) {
        getFaceConfig().setShowTrustLogos(shouldShowTrustLogos);
    }

    public void start(Context context, final FaceCaptureCompletionHandler callback) {
        HVFaceActivity.start(context, getFaceConfig(), new FaceCaptureCompletionHandler() {
            @RequiresApi(api = Build.VERSION_CODES.KITKAT)
            @Override
            public void onResult(HVError error, HVResponse hvResponse) {
                callback.onResult(error, hvResponse);
            }
        });
    }

    public void setUIStrings(Map<String, Object> uiStrings) {
        try {
            JSONObject uiStringsJsonObject = FLHVNetworkHelper.convertMapToJson(uiStrings);
            getFaceConfig().setCustomUIStrings(uiStringsJsonObject);
        } catch (JSONException e) {
            Log.e(TAG, e.getMessage());
        }
    }
}
