import 'package:flutter/services.dart';

import 'HVMethodCallStrings.dart';

class HVQRScanCapture {
  static const MethodChannel _hvQRScanCaptureChannel =
      const MethodChannel(HVMethodCallStrings.QR_SCAN_CAPTURE_CHANNEL);

  /// qrScanCaptureSetShouldShowInstructionPage() is set to true if instructions page has to be shown to the user
  ///     in the client's application
  ///
  /// params :
  ///     shouldShowInstructionPage : type - bool
  /// returns :
  ///     void
  static qrScanCaptureSetShouldShowInstructionPage(
      bool shouldShowInstructionPage) async {
    try {
      await _hvQRScanCaptureChannel.invokeMethod(
          HVMethodCallStrings.QR_SCAN_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE,
          {"shouldShowInstructionPage": shouldShowInstructionPage});
    } catch (e) {
      print(e.toString());
    }
  }

  /// qrScanCaptureStart() is used to start qr scan activity/view inside the client's application
  ///
  /// params :
  /// returns :
  ///     Future<Map>, contains the response of qrscan from native functions
  static Future<Map> qrScanCaptureStart() async {
    Map map = Map();
    try {
      map = await _hvQRScanCaptureChannel
          .invokeMethod(HVMethodCallStrings.QR_SCAN_CAPTURE_START);
    } catch (e) {
      print(e.toString());
    }
    return map;
  }
}
